component = require('component')
event = require('event')
fs = require('filesystem')
os = require('os')
term = require('term')

gpu = component.gpu

function simulate(cells)
  local isnil = false
  return function()
    if isnil then
      return nil
    end
    local newcells = {}
    local cellcount = 0
    for y=1,#cells do
      newcells[y] = {}
      for x=1,#cells[y] do
        local adj = 0
        for yo=-1,1 do
          for xo=-1,1 do
            if yo ~= 0 or xo ~= 0 then
              adj = adj + ((cells[y+yo] and cells[y+yo][x+xo]) and 1 or 0)
            end
          end
        end
        newcells[y][x] = ((cells[y][x] and adj == 2) or adj == 3)
        cellcount = cellcount + (newcells[y][x] and 1 or 0)
      end
    end
    isnil = cellcount == 0
    cells = newcells
    return cells
  end
end

function renderBoard(cells)
  for y=1,#cells do
    str = ''
    for x=1,#cells[y] do
      str = str..(cells[y][x] and 'X' or ' ')
    end
    gpu.set(1,y,str)
  end
end

function running()
  cells = simulate(cells)()
  renderBoard(cells)
end

function gamesave(path,data)
  local str = ''
  for y=1,#data do
    for x=1,#data[y] do
      str = str..(data[y][x] and 'x' or 'o')
    end
    str = str..'\n'
  end
  local file = fs.open(path,'w')
  file:write(string.sub(str,1,-2))
  file:close()
end

function gameload(path)
  local file = fs.open(path)
  local data = file:read(fs.size(path))
  file:close()
  local cells = {}
  local y = 0
  for str in string.gmatch(data,'[^\r\n]+') do
    y = y + 1
    cells[y] = {}
    for x=1,#str do
      if string.sub(str,x,x) == 'x' then
        cells[y][x] = true
      else
        cells[y][x] = false
      end
    end
  end
  return cells
end

local ow,oh = gpu.getResolution()

gpu.setResolution(68,22)

local w,h = gpu.getResolution()

cells = {}
local timer
local isStarted = false

gpu.set(1,h,'START STEP CLEAR'..string.rep(' ',w-21)..'CLOSE')

if fs.exists('gof.data') then
  cells = gameload('gof.data')
  renderBoard(cells)
else
  gpu.fill(1,1,w,h-1,' ')
  for y=1,h-1 do
    cells[y] = {}
    for x=1,w do
      cells[y][x] = false
    end
  end
end

while true do
  e,_,x,y,button = event.pullFiltered(function(e) return e == 'touch' or e == 'drag' or e == 'interrupted' end)
  if e == 'interrupted' then
    break
  elseif y == h and e == 'touch' then
    if x >= w-4 then
      break
    elseif x <= 4 + (not isStarted and 1 or 0) then
      isStarted = not isStarted
      gpu.set(1,h,isStarted and 'STOP      ' or 'START STEP')
      if isStarted then
        timer = event.timer(0.1,running,math.huge)
      else
        event.cancel(timer)
      end
    elseif not isStarted and x >= 7 and x <= 10 then
      cells = simulate(cells)()
      renderBoard(cells)
    elseif x >= 12 and x <= 16 then
      for y=1,h-1 do
        cells[y] = {}
        for x=1,w do
          cells[y][x] = false
        end
      end
      renderBoard(cells)
    end
  elseif y < h then
    cells[y][x] = button == 0
    gpu.set(x,y,button == 0 and 'X' or ' ')
  end
end
gamesave('gof.data',cells)
if timer then
  event.cancel(timer)
end
gpu.setResolution(ow,oh)
gpu.fill(1,1,ow,oh,' ')
term.setCursor(1,1)